(ns com.aurellem.gb.status
  (:use (com.aurellem.gb gb-driver util constants pokemon))
  (:import [com.aurellem.gb.gb_driver SaveState]))

(def status-name->status-code
  {:normal    (Integer/parseInt "00000000" 2)
   :sleep-1   (Integer/parseInt "00000001" 2)
   :sleep-2   (Integer/parseInt "00000010" 2)
   :sleep-3   (Integer/parseInt "00000011" 2)
   :sleep-4   (Integer/parseInt "00000100" 2)
   :sleep-5   (Integer/parseInt "00000101" 2)
   :sleep-6   (Integer/parseInt "00000111" 2)
   :poisoned  (Integer/parseInt "00001000" 2)
   :burned    (Integer/parseInt "00010000" 2)
   :frozen    (Integer/parseInt "00100000" 2)
   :paralyzed (Integer/parseInt "01000000" 2)})

(def status-code->status-name
  (zipmap (vals status-name->status-code)
          (keys status-name->status-code)))

(def pokemon-1-status-address 0xD16E)

(defn pokemon-status-address [poke-num]
  (+ pokemon-1-status-address
     (* poke-num pokemon-record-width)))

(defn read-status
  ([^SaveState state poke-num]
     (assert (<= 0 poke-num 5))
     (status-code->status-name
      (aget (memory state)
            (pokemon-status-address poke-num))))
  ([poke-num]
     (read-status @current-state poke-num)))
 
(defn give-status
  ([^SaveState state poke-num status]
     (assert (<= 0 poke-num 5))
     (let [status-code
           (status-name->status-code status)]
       (assert status-code)
       (set-memory state
                   (pokemon-status-address poke-num)
                   status-code)))
  ([poke-num status]
     (give-stat @current-state poke-num status))
  ([status]
     (give-stat @current-state 0 status)))

(defn give-status-all
  ([^SaveState state status]
     (reduce (fn [state num]
               (give-stat state num status))
             state
             (range (party-number state))))
  ([status]
     (give-stat-all @current-state status)))
